package com.soss.common.utils;

import com.google.zxing.BarcodeFormat;
import com.google.zxing.EncodeHintType;
import com.google.zxing.MultiFormatWriter;
import com.google.zxing.common.BitMatrix;
import com.google.zxing.qrcode.decoder.ErrorCorrectionLevel;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.Hashtable;

public class QRCodeUtil {
    private static int onColor = 0xFF000000;     //前景色
    private static int offColor = 0xFFFFFFFF;    //背景色
    private static int margin = 0;               //白边大小，取值范围0~4
    private static ErrorCorrectionLevel level = ErrorCorrectionLevel.L;  //二维码容错率

    /**
     * 生成二维码
     *
     * @param content //二维码内容
     */
    public static void generateQRImage(String content, String format, ByteArrayOutputStream byteArrayOutputStream, int width, int height) {
        Hashtable<EncodeHintType, Object> hints = new Hashtable<>();
        // 指定纠错等级
        hints.put(EncodeHintType.ERROR_CORRECTION, level);
        // 指定编码格式
        hints.put(EncodeHintType.CHARACTER_SET, "UTF-8");
        hints.put(EncodeHintType.MARGIN, margin);   //设置白边
        try {
            BitMatrix bitMatrix = new MultiFormatWriter().encode(content, BarcodeFormat.QR_CODE, width, height, hints);
            // bitMatrix = deleteWhite(bitMatrix);
            BufferedImage image = toBufferedImage(bitMatrix);
            // MatrixToImageWriter.writeToStream(bitMatrix, format, byteArrayOutputStream, config);
            ImageIO.write(image, format, byteArrayOutputStream); //生成二维码图片
//            ImageIO.write(image, format, new File("C:\\Users\\caiyo\\Desktop\\test.png")); //生成二维码图片
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void main(String[] args) {
        generateQRImage("test", "PNG", null, 151, 151);
    }

    /**
     * @param matrix                二维码矩阵相关
     * @param format                二维码图片格式
     * @param byteArrayOutputStream 二维码图片文件字节流
     * @param logoPath              logo路径
     * @throws IOException
     */
    public static void writeToFile(BitMatrix matrix, String format, ByteArrayOutputStream byteArrayOutputStream, String logoPath) throws IOException {
        BufferedImage image = toBufferedImage(matrix);
        //载入logo
        if (StringUtils.isNotEmpty(logoPath)) {
            int ratioWidth = image.getWidth() * 2 / 10;
            int ratioHeight = image.getHeight() * 2 / 10;
            Image img = ImageIO.read(new File(logoPath));
            int logoWidth = img.getWidth(null) > ratioWidth ? ratioWidth : img.getWidth(null);
            int logoHeight = img.getHeight(null) > ratioHeight ? ratioHeight : img.getHeight(null);
            int x = (image.getWidth() - logoWidth) / 2;
            int y = (image.getHeight() - logoHeight) / 2;

            Graphics2D gs = image.createGraphics();
            gs.drawImage(img, x, y, logoWidth, logoHeight, null);
            gs.setColor(Color.black);
            gs.setBackground(Color.WHITE);
            gs.dispose();
            img.flush();
        }
        ImageIO.write(image, format, byteArrayOutputStream); //生成二维码图片
    }

    public static BufferedImage toBufferedImage(BitMatrix matrix) {
        int width = matrix.getWidth();
        int height = matrix.getHeight();
        BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        for (int x = 0; x < width; x++) {
            for (int y = 0; y < height; y++) {
                image.setRGB(x, y, matrix.get(x, y) ? onColor : offColor);
            }
        }
        return image;
    }

    public static BitMatrix deleteWhite(BitMatrix matrix) {
        int[] rec = matrix.getEnclosingRectangle();
        int resWidth = rec[2] + 1;
        int resHeight = rec[3] + 1;
        BitMatrix resMatrix = new BitMatrix(resWidth, resHeight);
        resMatrix.clear();
        for (int i = 0; i < resWidth; i++) {
            for (int j = 0; j < resHeight; j++) {
                if (matrix.get(i + rec[0], j + rec[1]))
                    resMatrix.set(i, j);
            }
        }
        return resMatrix;
    }
}